@@ -5,13 +5,18 @@ import struct
 import zlib
 from collections import defaultdict
 from pathlib import Path
-
-import vdf
-from protontricks.cli import main
-from protontricks.steam import (APPINFO_STRUCT_HEADER, APPINFO_STRUCT_SECTION,
-                                SteamApp, get_appid_from_shortcut)
+from subprocess import run
 
 import pytest
+import vdf
+
+from protontricks.cli.desktop_install import \
+    cli as desktop_install_cli_entrypoint
+from protontricks.cli.launch import cli as launch_cli_entrypoint
+from protontricks.cli.main import cli as main_cli_entrypoint
+from protontricks.gui import get_gui_provider
+from protontricks.steam import (APPINFO_STRUCT_HEADER, APPINFO_STRUCT_SECTION,
+                                SteamApp, get_appid_from_shortcut)
 
 
 @pytest.fixture(scope="function", autouse=True)
@@ -23,27 +28,44 @@ def env_vars(monkeypatch):
     monkeypatch.setenv("STEAM_RUNTIME", "")
 
 
+@pytest.fixture(scope="function", autouse=True)
+def cleanup():
+    """
+    Miscellaneous cleanup tasks that need to be done before each test
+    """
+    # 'get_gui_provider' uses functools.lru_cache and needs to be cleared
+    # between tests
+    get_gui_provider.cache_clear()
+
+
 @pytest.fixture(scope="function", autouse=True)
 def home_dir(monkeypatch, tmp_path):
     """
     Fake home directory
     """
-    home_dir = tmp_path / "home" / "fakeuser"
-    home_dir.mkdir(parents=True)
+    home_dir_ = Path(str(tmp_path)) / "home" / "fakeuser"
+    home_dir_.mkdir(parents=True)
 
     # Create fake Winetricks executable
-    (home_dir / ".local" / "bin").mkdir(parents=True)
-    (home_dir / ".local" / "bin" / "winetricks").touch()
-    (home_dir / ".local" / "bin" / "winetricks").chmod(0o744)
+    (home_dir_ / ".local" / "bin").mkdir(parents=True)
+    (home_dir_ / ".local" / "bin" / "winetricks").touch()
+    (home_dir_ / ".local" / "bin" / "winetricks").chmod(0o744)
 
-    monkeypatch.setenv("HOME", str(home_dir))
+    # Create fake YAD and Zenity executable
+    (home_dir_ / ".local" / "bin" / "zenity").touch()
+    (home_dir_ / ".local" / "bin" / "zenity").chmod(0o744)
+
+    (home_dir_ / ".local" / "bin" / "yad").touch()
+    (home_dir_ / ".local" / "bin" / "yad").chmod(0o744)
+
+    monkeypatch.setenv("HOME", str(home_dir_))
 
     # Set PATH to point only towards the fake home directory
     # This helps prevent the system-wide binaries from messing with tests
     # where we test for absence of executables such as 'winetricks'
-    monkeypatch.setenv("PATH", str(home_dir / ".local" / "bin"))
+    monkeypatch.setenv("PATH", str(home_dir_ / ".local" / "bin"))
 
-    yield home_dir
+    yield home_dir_
 
 
 @pytest.fixture(scope="function", autouse=True)
@@ -56,7 +78,7 @@ def steam_dir(home_dir):
 
     (steam_path / "root" / "compatibilitytools.d").mkdir(parents=True)
 
-    (steam_path / "steam" / "appcache").mkdir(parents=True)
+    (steam_path / "steam" / "appcache" / "librarycache").mkdir(parents=True)
     (steam_path / "steam" / "config").mkdir(parents=True)
     (steam_path / "steam" / "steamapps").mkdir(parents=True)
 
@@ -134,7 +156,7 @@ def shortcut_factory(steam_dir, steam_user):
     """
     shortcuts_by_user = defaultdict(list)
 
-    def func(install_dir, name, steamid64=None):
+    def func(install_dir, name, steamid64=None, appid_in_vdf=False):
         if not steamid64:
             steamid64 = steam_user
 
@@ -168,6 +190,11 @@ def shortcut_factory(steam_dir, steam_user):
             result = result | 0x80000000
             shortcut_id = (result << 32) | 0x02000000
 
+            if appid_in_vdf:
+                # Store the app ID in `shortcuts.vdf`. This is similar
+                # in behavior to newer Steam releases.
+                entry["appid"] = ~(result ^ 0xffffffff)
+
             data["shortcuts"][str(shortcut_id)] = entry
 
         shortcut_path.write_bytes(vdf.binary_dumps(data))
@@ -305,13 +332,23 @@ def steam_app_factory(steam_dir, steam_config_path):
     """
     def func(
             name, appid, compat_tool_name=None, library_dir=None,
-            add_prefix=True):
+            add_prefix=True, required_tool_app=None):
         if not library_dir:
             steamapps_dir = steam_dir / "steamapps"
         else:
             steamapps_dir = library_dir / "steamapps"
 
-        (steamapps_dir / "common" / name).mkdir(parents=True)
+        install_path = steamapps_dir / "common" / name
+        install_path.mkdir(parents=True)
+
+        if required_tool_app:
+            (install_path / "toolmanifest.vdf").write_text(
+                vdf.dumps({
+                    "manifest": {
+                        "require_tool_appid": required_tool_app.appid
+                    }
+                })
+            )
 
         (steamapps_dir / "appmanifest_{}.acf".format(appid)).write_text(
             vdf.dumps({
@@ -341,7 +378,7 @@ def steam_app_factory(steam_dir, steam_config_path):
                 }
             steam_config_path.write_text(vdf.dumps(steam_config))
 
-        return SteamApp(
+        steam_app = SteamApp(
             name=name,
             appid=appid,
             install_path=str(steamapps_dir / "common" / name),
@@ -350,6 +387,14 @@ def steam_app_factory(steam_dir, steam_config_path):
                 / "pfx"
             )
         )
+        if required_tool_app:
+            # In actual code, `required_tool_app` attribute is populated later
+            # when we have retrieved all Steam apps and can find the
+            # corresponding app using its app ID
+            steam_app.required_tool_app = required_tool_app
+            steam_app.required_tool_appid = required_tool_app.appid
+
+        return steam_app
 
     return func
 
@@ -361,14 +406,20 @@ def proton_factory(steam_app_factory, appinfo_factory, steam_config_path):
     """
     def func(
             name, appid, compat_tool_name, is_default_proton=True,
-            library_dir=None):
+            library_dir=None, required_tool_app=None):
         steam_app = steam_app_factory(
-            name=name, appid=appid, library_dir=library_dir
+            name=name, appid=appid, library_dir=library_dir,
+            required_tool_app=required_tool_app
         )
         shutil.rmtree(str(Path(steam_app.prefix_path).parent))
         steam_app.prefix_path = None
 
-        (Path(steam_app.install_path) / "proton").touch()
+        install_path = Path(steam_app.install_path)
+
+        (install_path / "proton").touch()
+        (install_path / "dist" / "bin").mkdir(parents=True)
+        (install_path / "dist" / "bin" / "wine").touch()
+        (install_path / "dist" / "bin" / "wineserver").touch()
 
         # Update config
         if is_default_proton:
@@ -392,12 +443,54 @@ def proton_factory(steam_app_factory, appinfo_factory, steam_config_path):
     return func
 
 
+@pytest.fixture(scope="function")
+def runtime_app_factory(steam_app_factory, appinfo_factory, steam_config_path):
+    """
+    Factory function to add fake Steam Runtimes that are installed as Steam
+    apps
+    """
+    def func(name, appid, runtime_dir_name, library_dir=None):
+        runtime_app = steam_app_factory(
+            name=name, appid=appid, library_dir=library_dir, add_prefix=False
+        )
+
+        install_path = Path(runtime_app.install_path)
+
+        runtime_root_path = install_path / runtime_dir_name / "files"
+        (runtime_root_path / "lib" / "i386-linux-gnu").mkdir(parents=True)
+        (runtime_root_path / "lib" / "x86_64-linux-gnu").mkdir(parents=True)
+
+        (install_path / "run.sh").touch()
+
+        (install_path / "toolmanifest.vdf").write_text(
+            vdf.dumps({
+                "manifest": {"commandline": "/run.sh --"}
+            })
+        )
+
+        return runtime_app
+
+    return func
+
+
+@pytest.fixture(scope="function")
+def steam_runtime_soldier(runtime_app_factory):
+    """
+    Fake Steam Runtime Soldier installation
+    """
+    return runtime_app_factory(
+        name="Steam Linux Runtime - Soldier",
+        appid=1391110,
+        runtime_dir_name="soldier"
+    )
+
+
 @pytest.fixture(scope="function")
 def custom_proton_factory(steam_dir):
     """
     Factory function to add fake custom Proton installations
     """
-    def func(name, compat_tool_dir=None):
+    def func(name, compat_tool_dir=None, required_tool_app=None):
         if not compat_tool_dir:
             compat_tool_dir = \
                 steam_dir.parent / "root" / "compatibilitytools.d" / name
@@ -406,6 +499,9 @@ def custom_proton_factory(steam_dir):
         compat_tool_dir.mkdir(parents=True, exist_ok=True)
         (compat_tool_dir / "proton").touch()
         (compat_tool_dir / "proton").chmod(0o744)
+        (compat_tool_dir / "dist" / "bin").mkdir(parents=True)
+        (compat_tool_dir / "dist" / "bin" / "wine").touch()
+        (compat_tool_dir / "dist" / "bin" / "wineserver").touch()
         (compat_tool_dir / "compatibilitytool.vdf").write_text(
             vdf.dumps({
                 "compatibilitytools": {
@@ -421,6 +517,19 @@ def custom_proton_factory(steam_dir):
             })
         )
 
+        if required_tool_app:
+            (compat_tool_dir / "toolmanifest.vdf").write_text(
+                vdf.dumps({
+                    "manifest": {
+                        "require_tool_appid": required_tool_app.appid
+                    }
+                })
+            )
+        else:
+            (compat_tool_dir / "toolmanifest.vdf").write_text(
+                vdf.dumps({"manifest": {}})
+            )
+
         return SteamApp(
             name=name,
             install_path=str(compat_tool_dir)
@@ -445,8 +554,8 @@ def steam_library_factory(steam_dir, steam_libraryfolders_path, tmp_path):
     """
     Factory function to add fake Steam library folders
     """
-    def func(name):
-        library_dir = tmp_path / "mnt" / name
+    def func(name, new_struct=False):
+        library_dir = Path(str(tmp_path)) / "mnt" / name
         library_dir.mkdir(parents=True)
 
         # Update libraryfolders.vdf with the new library folder
@@ -457,9 +566,18 @@ def steam_library_factory(steam_dir, steam_libraryfolders_path, tmp_path):
         # Each new library adds a new entry into the config file with the
         # field name that starts from 1 and increases with each new library
         # folder.
+        # Newer Steam releases stores the library entry in a dict, while
+        # older releases just store the full path as the field value
         library_id = len(libraryfolders_config["LibraryFolders"].keys()) - 1
-        libraryfolders_config["LibraryFolders"][str(library_id)] = \
-            str(library_dir)
+        if new_struct:
+            libraryfolders_config["LibraryFolders"][str(library_id)] = {
+                "path": str(library_dir),
+                "label": "",
+                "mounted": "1"
+            }
+        else:
+            libraryfolders_config["LibraryFolders"][str(library_id)] = \
+                str(library_dir)
 
         steam_libraryfolders_path.write_text(vdf.dumps(libraryfolders_config))
 
@@ -486,26 +604,30 @@ class MockResult:
         self.stdout = stdout
 
 
-@pytest.fixture(scope="function", autouse=True)
-def zenity(monkeypatch):
+@pytest.fixture(scope="function")
+def gui_provider(monkeypatch):
     """
-    Monkeypatch the subprocess.run to store the args passed to the zenity
+    Monkeypatch the subprocess.run to store the args passed to the yad/zenity
     command and to manipulate the output of the command
     """
-    mock_zenity = MockSubprocess()
+    mock_gui_provider = MockSubprocess()
 
     def mock_subprocess_run(args, **kwargs):
-        mock_zenity.args = args
-        mock_zenity.kwargs = kwargs
+        mock_gui_provider.args = args
+        mock_gui_provider.kwargs = kwargs
 
-        return MockResult(stdout=mock_zenity.mock_stdout.encode("utf-8"))
+        return MockResult(stdout=mock_gui_provider.mock_stdout.encode("utf-8"))
 
     monkeypatch.setattr(
         "protontricks.gui.run",
         mock_subprocess_run
     )
+    monkeypatch.setattr(
+        "protontricks.cli.util.run",
+        mock_subprocess_run
+    )
 
-    yield mock_zenity
+    yield mock_gui_provider
 
 
 @pytest.fixture(scope="function", autouse=True)
@@ -517,6 +639,10 @@ def command(monkeypatch):
     mock_command = MockSubprocess()
 
     def mock_subprocess_run(args, **kwargs):
+        # Don't mock "/sbin/ldconfig"
+        if args[0] == "/sbin/ldconfig":
+            return run(args, **kwargs)
+
         mock_command.args = args
         mock_command.kwargs = kwargs
         mock_command.env = os.environ.copy()
@@ -527,12 +653,15 @@ def command(monkeypatch):
         "protontricks.util.run",
         mock_subprocess_run
     )
+    monkeypatch.setattr(
+        "protontricks.cli.desktop_install.run",
+        mock_subprocess_run
+    )
 
     yield mock_command
 
 
-@pytest.fixture(scope="function")
-def cli(monkeypatch, capsys):
+def _run_cli(monkeypatch, capsys, cli_func):
     """
     Run protontricks with the given arguments and environment variables
     and return the output
@@ -550,7 +679,7 @@ def cli(monkeypatch, capsys):
                 monkeypatch_ctx.setenv(name, val)
 
             try:
-                main(args)
+                cli_func(args)
             except SystemExit:
                 if expect_exit:
                     system_exit = True
@@ -568,3 +697,30 @@ def cli(monkeypatch, capsys):
             return stdout
 
     return func
+
+
+@pytest.fixture(scope="function")
+def cli(monkeypatch, capsys):
+    """
+    Run `protontricks` with the given arguments and environment variables,
+    and return the output
+    """
+    return _run_cli(monkeypatch, capsys, main_cli_entrypoint)
+
+
+@pytest.fixture(scope="function")
+def launch_cli(monkeypatch, capsys):
+    """
+    Run `protontricks-launch` with the given arguments and environment
+    variables, and return the output
+    """
+    return _run_cli(monkeypatch, capsys, launch_cli_entrypoint)
+
+
+@pytest.fixture(scope="function")
+def desktop_install_cli(monkeypatch, capsys):
+    """
+    Run `protontricks-desktop-install` with the given arguments and environment
+    variables, and return the output
+    """
+    return _run_cli(monkeypatch, capsys, desktop_install_cli_entrypoint)
